#!/bin/sh

all_modules=$(sh build.sh -L)

branch="__autogen-patch-prefix__"
outdir="autogen-patch-prefix_patches"

mkdir -p "$outdir"
outdir_abs=$(realpath -e "$outdir")

for module in $all_modules ; do
  (
  echo
  echo "Adding autogen patch prefix to '$module' ..."
  module_name="${module##*/}"
  module_subjectname="$module_name"
  case "$module" in
    app/*|font/*|util/*|xcb/*)
      module_subjectname="$module"
      ;;
  esac
  module_filename=$(echo "$module" | sed -e 's/[/]/_/')

  cd "$module" || exit 1
  git status || exit 1

  git checkout master || exit 1
  grep -q subjectPrefix autogen.sh && continue

  git branch -D "$branch" 2>/dev/null
  git checkout -b "$branch" || exit 1

  configure_line_re="NOCONFIGURE"
  grep -q "$configure_line_re" autogen.sh || configure_line_re="configure"

  sed -i -e '
    /'"$configure_line_re"'/ s~^~git config --local --get format.subjectPrefix >/dev/null 2>\&1 ||\n    git config --local format.subjectPrefix "PATCH '"$module_subjectname"'"\n\n~
  ' autogen.sh

  git add autogen.sh
  git commit -m "autogen: add default patch prefix"

  outfile="$outdir_abs"/"$module_filename".patch
  git format-patch HEAD^ \
    --subject-prefix="PATCH $module_subjectname" \
    --signoff \
    --stdout \
    > "$outfile" || exit 1

  if grep -q "0 insert" "$outfile" ; then
    echo
    echo "Error patching $module !"
    exit 1
  fi

  git checkout master
  git branch -D "$branch"
  ) || exit 1
done
